@@ -7,6 +7,8 @@
 // specific language governing permissions and limitations relating to use of the SAFE Network
 // Software.
 
+use crate::connection_pool::ConnId;
+
 use super::error::Error;
 use super::wire_msg::WireMsg;
 #[cfg(not(feature = "no-igd"))]
@@ -68,22 +70,21 @@ impl IncomingConnections {
 /// Endpoint instance which can be used to create connections to peers,
 /// and listen to incoming messages from other peers.
 #[derive(Clone)]
-pub struct Endpoint {
+pub struct Endpoint<I: ConnId> {
     local_addr: SocketAddr,
     public_addr: Option<SocketAddr>,
     quic_endpoint: quinn::Endpoint,
     message_tx: MpscSender<(SocketAddr, Bytes)>,
     disconnection_tx: MpscSender<SocketAddr>,
-
     client_cfg: quinn::ClientConfig,
     bootstrap_nodes: Vec<SocketAddr>,
     qp2p_config: Config,
     termination_tx: Sender<()>,
-    connection_pool: ConnectionPool,
+    connection_pool: ConnectionPool<I>,
     connection_deduplicator: ConnectionDeduplicator,
 }
 
-impl std::fmt::Debug for Endpoint {
+impl<I: ConnId> std::fmt::Debug for Endpoint<I> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("Endpoint")
             .field("local_addr", &self.local_addr)
@@ -93,7 +94,7 @@ impl std::fmt::Debug for Endpoint {
     }
 }
 
-impl Endpoint {
+impl<I: ConnId> Endpoint<I> {
     pub(crate) async fn new(
         quic_endpoint: quinn::Endpoint,
         quic_incoming: quinn::Incoming,
@@ -140,7 +141,7 @@ impl Endpoint {
                 info!("Verifying provided public IP address");
                 endpoint.connect_to(contact).await?;
                 let connection = endpoint
-                    .get_connection(&contact)
+                    .get_connection(contact)
                     .await
                     .ok_or(Error::MissingConnection)?;
                 let (mut send, mut recv) = connection.open_bi().await?;
@@ -338,7 +339,7 @@ impl Endpoint {
     /// from the pool and the subsequent call to `connect_to` is guaranteed to reopen new connection
     /// too.
     pub async fn connect_to(&self, node_addr: &SocketAddr) -> Result<()> {
-        if self.connection_pool.has(node_addr).await {
+        if self.connection_pool.has_addr(node_addr).await {
             trace!("We are already connected to this peer: {}", node_addr);
             return Ok(());
         }
@@ -353,7 +354,7 @@ impl Endpoint {
 
         trace!("Successfully connected to peer: {}", node_addr);
 
-        self.add_new_connection_to_pool(final_conn).await;
+        self.add_new_connection_to_pool(final_conn).await?;
 
         self.connection_deduplicator
             .complete(node_addr, Ok(()))
@@ -451,10 +452,15 @@ impl Endpoint {
         Ok(new_connection)
     }
 
-    pub(crate) async fn add_new_connection_to_pool(&self, conn: quinn::NewConnection) {
+    pub(crate) async fn add_new_connection_to_pool(
+        &self,
+        conn: quinn::NewConnection,
+    ) -> Result<()> {
+        let id = ConnId::generate(&conn.connection.remote_address())
+            .map_err(|err| Error::ConnectionIdGeneration(err.to_string()))?;
         let guard = self
             .connection_pool
-            .insert(conn.connection.remote_address(), conn.connection)
+            .insert(id, conn.connection.remote_address(), conn.connection)
             .await;
 
         listen_for_incoming_messages(
@@ -465,11 +471,12 @@ impl Endpoint {
             self.disconnection_tx.clone(),
             self.clone(),
         );
+        Ok(())
     }
 
     /// Get an existing connection for the peer address.
-    pub(crate) async fn get_connection(&self, peer_addr: &SocketAddr) -> Option<Connection> {
-        if let Some((conn, guard)) = self.connection_pool.get(peer_addr).await {
+    pub(crate) async fn get_connection(&self, peer_addr: &SocketAddr) -> Option<Connection<I>> {
+        if let Some((conn, guard)) = self.connection_pool.get_by_addr(peer_addr).await {
             trace!("Connection exists in the connection pool: {}", peer_addr);
             Some(Connection::new(conn, guard))
         } else {
@@ -477,6 +484,22 @@ impl Endpoint {
         }
     }
 
+    /// Get the connection ID of an existing connection with the provided socket address
+    pub async fn get_connection_id(&self, addr: &SocketAddr) -> Option<I> {
+        self.connection_pool
+            .get_by_addr(addr)
+            .await
+            .map(|(_, remover)| remover.id())
+    }
+
+    /// Get the SocketAddr of a connection using the connection ID
+    pub async fn get_socket_addr_by_id(&self, addr: &I) -> Option<SocketAddr> {
+        self.connection_pool
+            .get_by_id(addr)
+            .await
+            .map(|(_, remover)| *remover.remote_addr())
+    }
+
     /// Open a bi-directional peer with a given peer
     pub async fn open_bidirectional_stream(
         &self,
@@ -563,10 +586,10 @@ impl Endpoint {
         &self.bootstrap_nodes
     }
 
-    fn retry<I, E, Fn, Fut>(&self, op: Fn) -> impl futures::Future<Output = Result<I, E>>
+    fn retry<R, E, Fn, Fut>(&self, op: Fn) -> impl futures::Future<Output = Result<R, E>>
     where
         Fn: FnMut() -> Fut,
-        Fut: futures::Future<Output = Result<I, backoff::Error<E>>>,
+        Fut: futures::Future<Output = Result<R, backoff::Error<E>>>,
     {
         let backoff = ExponentialBackoff {
             max_elapsed_time: Some(Duration::from_millis(self.qp2p_config.retry_duration_msec)),
